#ifndef __ALGORITHMAN__BIGINT_H
    #define __ALGORITHMAN__BIGINT_H

    #include<iostream>
    #include<vector>
    #include<string>
    
    namespace AlgorithMan
    {
        
        // bits[0] is the least significant bit
        
        class bigint
        {
            friend std::ostream& operator<<(std::ostream& os, const bigint&);
            friend std::istream& operator>>(std::istream& is, bigint&);
            protected:
                std::vector<bool> bits;
                void normalize(); // removes leading zeros
                void divmod(const bigint& x, bigint& div, bigint& mod) const;
                
            public:
                bigint();
                bigint(unsigned int value);
                bigint(const std::string& value);
                bigint(const bigint&); // copy constructor
                ~bigint();
                
                static bigint parse(const std::string&);
                std::string to_string(int base=10) const;
            
                bigint operator=(const bigint&);
                bigint operator+(const bigint&) const;
                bigint operator-(const bigint&) const;
                bigint operator*(const bigint&) const;
                bigint operator/(const bigint&) const;
                bigint operator%(const bigint&) const;
                bigint operator<<(const bigint&) const;
                bigint operator>>(const bigint&) const;
                
                void operator+=(const bigint&);
                void operator-=(const bigint&);
                void operator*=(const bigint&);
                void operator/=(const bigint&);
                void operator%=(const bigint&);
                void operator<<=(const bigint&);
                void operator>>=(const bigint&);

                bool operator==(const bigint&) const;
                bool operator!=(const bigint&) const;
                bool operator< (const bigint&) const;
                bool operator<=(const bigint&) const;
                bool operator> (const bigint&) const;
                bool operator>=(const bigint&) const;
        };
        
        std::ostream& operator<<(std::ostream& os, const bigint&);
        std::istream& operator>>(std::istream& is, bigint&);
    }    
    
#endif